@@ -5,7 +5,7 @@
 ;; Author: Protesilaos Stavrou <info@protesilaos.com>
 ;; Maintainer: Protesilaos Stavrou <info@protesilaos.com>
 ;; URL: https://github.com/protesilaos/denote-journal
-;; Version: 0.0.0.1
+;; Version: 0.1.0
 ;; Package-Requires: ((emacs "28.1") (denote "3.1.0"))
 
 ;; This file is NOT part of GNU Emacs.
@@ -32,12 +32,16 @@
 ;;; Code:
 
 (require 'denote)
+(require 'calendar)
+(eval-when-compile (require 'cl-lib))
 
 (defgroup denote-journal nil
   "Convenience functions for daily journaling with Denote."
   :group 'denote
   :link '(info-link "(denote) Top")
-  :link '(url-link :tag "Homepage" "https://protesilaos.com/emacs/denote"))
+  :link '(info-link "(denote-journal) Top")
+  :link '(url-link :tag "Denote homepage" "https://protesilaos.com/emacs/denote")
+  :link '(url-link :tag "Denote Journal homepage" "https://protesilaos.com/emacs/denote-journal"))
 
 (defcustom denote-journal-directory
   (expand-file-name "journal" denote-directory)
@@ -62,21 +66,30 @@ It is used by `denote-journal-new-entry' (or related)."
 
 (defcustom denote-journal-title-format 'day-date-month-year-24h
   "Date format to construct the title with `denote-journal-new-entry'.
-The value is either a symbol or an arbitrary string that is
-passed to `format-time-string' (consult its documentation for the
-technicalities).
+The value it can take is either nil, a
+custom string, or a symbol:
 
-Acceptable symbols and their corresponding styles are:
+- When `denote-journal-title-format' is set to a nil value, then new
+  journal entries always prompt for a title.  Users will want this if
+  they prefer to journal using a given theme for the day rather than
+  the date itself (e.g. instead of \"1st of April 2025\" they may prefer
+  something like \"Early Spring at the hut\").
 
-| Symbol                  | Style                             |
-|-------------------------+-----------------------------------|
-| day                     | Monday                            |
-| day-date-month-year     | Monday 19 September 2023          |
-| day-date-month-year-24h | Monday 19 September 2023 20:49    |
-| day-date-month-year-12h | Monday 19 September 2023 08:49 PM |
+- When `denote-journal-title-format' is set to an empty or blank
+  string (string with only spaces), then new journal entries will not
+  use a file title.
 
-With a nil value, make `denote-journal-new-entry' prompt
-for a title."
+- When `denote-journal-title-format' is set to a symbol, it is one
+  among `day' (results in a title like \"Tuesday\"), `day-date-month-year'
+  (for a result like \"Tuesday 1 April 2025\"), `day-date-month-year-24h'
+  (for \"Tuesday 1 April 2025 13:46\"), or `day-date-month-year-12h'
+  (e.g. \"Tuesday 1 April 2025 02:46 PM\").
+
+- When `denote-journal-title-format' is set to a string, it is used
+  literally except for any \"format specifiers\", as interpreted by the
+  function `format-time-string', which are replaced by their given
+  date component.  For example, the `\"Week %V on %A %e %B %Y at %H:%M\"''
+  will yield a title like \"Week 14 on 1 April 2025 at 13:48\"."
   :group 'denote-journal
   :type '(choice
           (const :tag "Prompt for title with `denote-journal-new-entry'" nil)
@@ -133,32 +146,35 @@ journal entry (refer to the `tmr' package on GNU ELPA)."
        (string-match-p (denote-journal--keyword-regex) (file-name-nondirectory filename))))
 
 (defun denote-journal-daily--title-format (&optional date)
-  "Return present date in `denote-journal-title-format' or prompt for title.
-With optional DATE, use it instead of the present date.  DATE has
-the same format as that returned by `current-time'."
-  (format-time-string
-   (if (and denote-journal-title-format
-            (stringp denote-journal-title-format))
-       denote-journal-title-format
-     (pcase denote-journal-title-format
-       ('day "%A")
-       ('day-date-month-year "%A %e %B %Y")
-       ('day-date-month-year-24h "%A %e %B %Y %H:%M")
-       ('day-date-month-year-12h "%A %e %B %Y %I:%M %^p")
-       (_ (denote-title-prompt (format-time-string "%F" date)))))
-   date))
+  "Return appropriate value for `denote-journal-title-format'.
+With optional DATE, use it instead of the present date wherever
+relevant.  DATE has the same format as that returned by `current-time'."
+  (let ((specifiers (pcase denote-journal-title-format
+                      ((pred null)
+                       (cons
+                        (denote-title-prompt (format-time-string "%F" date) "New journal file TITLE")
+                        :skip))
+                      ((and (pred stringp) (pred string-blank-p))
+                       (cons "" :skip))
+                      ((pred stringp) denote-journal-title-format)
+                      ('day "%A")
+                      ('day-date-month-year "%A %e %B %Y")
+                      ('day-date-month-year-24h "%A %e %B %Y %H:%M")
+                      ('day-date-month-year-12h "%A %e %B %Y %I:%M %^p"))))
+    (if (consp specifiers)
+        (car specifiers)
+      (format-time-string specifiers date))))
 
 (defun denote-journal--get-template ()
   "Return template that has `journal' key in `denote-templates'.
 If no template with `journal' key exists but `denote-templates'
 is non-nil, prompt the user for a template among
-`denote-templates'.  Else return nil.
-
-Also see `denote-journal-new-entry'."
-  (if-let* ((template (alist-get 'journal denote-templates)))
-      template
-    (when denote-templates
-      (denote-template-prompt))))
+`denote-templates'.  Else return nil."
+  ;; FIXME 2025-04-02: Here we assume that `denote-templates' is an
+  ;; alist.  Maybe we need to be more careful.
+  (when denote-templates
+    (or (alist-get 'journal denote-templates)
+        (denote-template-prompt))))
 
 ;;;###autoload
 (defun denote-journal-new-entry (&optional date)
@@ -192,42 +208,44 @@ DATE has the same format as that returned by `denote-valid-date-p'."
          (id-index (seq-position order 'identifier))
          (kw-index (seq-position order 'keywords)))
     (if (> kw-index id-index)
-        (format "%s.*?_%s" identifier (denote-journal--keyword-regex))
-      (format "_%s.*?@@%s" (denote-journal--keyword-regex) identifier))))
+        (format "%s.*?%s" identifier (denote-journal--keyword-regex))
+      (format "%s.*?@@%s" (denote-journal--keyword-regex) identifier))))
 
 (defun denote-journal--entry-today (&optional date)
   "Return list of files matching a journal for today or optional DATE.
 DATE has the same format as that returned by `denote-valid-date-p'."
-  (denote-directory-files (denote-journal--filename-date-regexp date)))
+  (let ((denote-directory (denote-journal-directory)))
+    (denote-directory-files (denote-journal--filename-date-regexp date))))
 
-(define-obsolete-function-alias
-  'denote-journal-extra-path-to-new-or-existing-entry
-  'denote-journal-path-to-new-or-existing-entry
-  "3.2.0")
+(defun denote-journal-select-file-prompt (files)
+  "Prompt for file among FILES if >1, else return the `car'.
+Perform the operation relative to the variable `denote-journal-directory'."
+  (let* ((default-directory (denote-journal-directory))
+         (denote-directory default-directory)
+         (relative-files (mapcar #'denote-get-file-name-relative-to-denote-directory files))
+         (file (if (> (length files) 1)
+                   (completing-read "Select journal entry: " (denote--completion-table 'file relative-files) nil t)
+                 (car relative-files))))
+    (concat denote-directory file)))
 
 ;;;###autoload
 (defun denote-journal-path-to-new-or-existing-entry (&optional date)
   "Return path to existing or new journal file.
 With optional DATE, do it for that date, else do it for today.  DATE is
 a string and has the same format as that covered in the documentation of
-the `denote' function.  It is internally processed by
-`denote-valid-date-p'.
+the `denote' function.  It is internally processed by `denote-valid-date-p'.
 
 If there are multiple journal entries for the date, prompt for one among
 them using minibuffer completion.  If there is only one, return it.  If
 there is no journal entry, create it."
   (let* ((internal-date (or (denote-valid-date-p date) (current-time)))
          (files (denote-journal--entry-today internal-date)))
-    (cond
-     ((length> files 1)
-      (completing-read "Select journal entry: " files nil t))
-     (files
-      (car files))
-     (t
+    (if files
+        (denote-journal-select-file-prompt files)
       (save-window-excursion
         (denote-journal-new-entry date)
         (save-buffer)
-        (buffer-file-name))))))
+        (buffer-file-name)))))
 
 ;;;###autoload
 (defun denote-journal-new-or-existing-entry (&optional date)
@@ -285,5 +303,109 @@ file's title.  This has the same meaning as in `denote-link'."
                  (denote-get-link-description path)
                  id-only)))
 
+;;;; Integration with the `calendar'
+
+(defface denote-journal-calendar '((t :box (:line-width (-1 . -1))))
+  "Face to mark a Denote journal entry in the `calendar'.")
+
+(defun denote-journal-calendar--file-to-date (file)
+  "Convert FILE to calendar date by interpreting its identifier."
+  (when-let* ((identifier (denote-retrieve-filename-identifier file))
+              (date (denote--id-to-date identifier))
+              (numbers (mapcar #'string-to-number (split-string date "-"))))
+    (pcase-let ((`(,year ,month ,day) numbers))
+      (list month day year))))
+
+(defun denote-journal-calendar--get-files (calendar-date)
+  "Return files around CALENDAR-DATE in variable `denote-journal-keyword'."
+  (pcase-let* ((denote-directory (denote-journal-directory))
+               (interval (calendar-interval
+                          displayed-month displayed-year ; These are local to the `calendar'
+                          (calendar-extract-month calendar-date)
+                          (calendar-extract-year calendar-date)))
+               (`(,current-month ,_ ,current-year) calendar-date)
+               (`(,previous-month . ,previous-year) (calendar-increment-month-cons (- interval 1)))
+               (`(,next-month . ,next-year) (calendar-increment-month-cons (+ interval 1)))
+               (years (list previous-year current-year next-year))
+               (months (list previous-month current-month next-month))
+               (time-regexp (concat (regexp-opt (mapcar #'number-to-string years))
+                                    (regexp-opt (mapcar (lambda (number) (format "%02d" number)) months))))
+               (keyword-regexp (denote-journal--keyword-regex)))
+    (denote-directory-files
+     ;; NOTE 2025-03-31: This complex regular expression is to account
+     ;; for `denote-file-name-components-order'.  We should probably
+     ;; have something in `denote.el' to do this fancy stuff, though
+     ;; this is the first time I have a use-case for it.
+     (format "\\(%1$s.*%2$s\\)\\|\\(%2$s.*%1$s\\)" time-regexp keyword-regexp))))
+
+(defun denote-journal-calendar-mark-dates ()
+  "Mark all visible days in the `calendar' for which there is a Denote journal entry."
+  (interactive)
+  (when-let* ((date (calendar-cursor-to-date))
+              (files (denote-journal-calendar--get-files date))
+              (dates (delq nil (mapcar #'denote-journal-calendar--file-to-date files))))
+    (dolist (date dates)
+      (calendar-mark-visible-date date 'denote-journal-calendar))))
+
+(defun denote-journal-calendar--date-to-time (calendar-date)
+  "Return internal time of `calendar' CALENDAR-DATE.
+CALENDAR-DATE is a list of three numbers, in the form of (MONTH DAY YEAR)."
+  (pcase-let ((`(,month ,day ,year) calendar-date)
+              (time (format-time-string "%T")))
+    (date-to-time (format "%s-%02d-%02d %s" year month day time))))
+
+(defun denote-journal-calendar--date-to-identifier (calendar-date)
+  "Return path to Denote journal entry corresponding to CALENDAR-DATE.
+CALENDAR-DATE is a list of three numbers, in the form of (MONTH DAY YEAR)."
+  (when-let* ((date (denote-journal-calendar--date-to-time calendar-date)))
+    (denote-journal--entry-today date)))
+
+(defun denote-journal-calendar-find-file ()
+  "Show the Denote journal entry for the `calendar' date at point.
+If there are more than one files, prompt with completion to select one
+among them."
+  (declare (interactive-only t))
+  (interactive nil calendar-mode)
+  (unless (derived-mode-p 'calendar-mode)
+    (user-error "Only use this command inside the `calendar'"))
+  (when-let* ((calendar-date (calendar-cursor-to-date)))
+    (if-let* ((files (denote-journal-calendar--date-to-identifier calendar-date))
+              (file (denote-journal-select-file-prompt files)))
+        (funcall denote-open-link-function file)
+      (user-error "No Denote journal entry for this date"))))
+
+(defun denote-journal-calendar-new-or-existing ()
+  "Like `denote-journal-new-or-existing-entry' for the `calendar' date at point."
+  (declare (interactive-only t))
+  (interactive nil calendar-mode)
+  (unless (derived-mode-p 'calendar-mode)
+    (user-error "Only use this command inside the `calendar'"))
+  (when-let* ((calendar-date (calendar-cursor-to-date)))
+    (if-let* ((internal (denote-journal-calendar--date-to-time calendar-date)))
+        (progn
+          (calendar-mark-visible-date calendar-date 'denote-journal-calendar)
+          ;; Do not use the same `calendar' window...
+          (cl-letf (((symbol-function #'find-file) denote-open-link-function))
+            (denote-journal-new-or-existing-entry internal)))
+      (user-error "No Denote journal entry for this date"))))
+
+(defvar denote-journal-calendar-mode-map
+  (let ((map (make-sparse-keymap)))
+    (define-key map "N" #'denote-journal-calendar-new-or-existing)
+    (define-key map "F" #'denote-journal-calendar-find-file)
+    map)
+  "Key map for `denote-journal-calendar-mode'.")
+
+(define-minor-mode denote-journal-calendar-mode
+  "Mark Denote journal entries using `denote-journal-calendar' face.
+Add the function `denote-journal-calendar-mode' to the
+`calendar-mode-hook' for changes to take effect."
+  :global nil
+  (if denote-journal-calendar-mode
+      (dolist (hook '(calendar-today-visible-hook calendar-today-invisible-hook))
+        (add-hook hook #'denote-journal-calendar-mark-dates nil :local))
+    (dolist (hook '(calendar-today-visible-hook calendar-today-invisible-hook))
+      (remove-hook hook #'denote-journal-calendar-mark-dates :local))))
+
 (provide 'denote-journal)
 ;;; denote-journal.el ends here
